/*
 * 
 *
 * 
 */

#include <stdlib.h>
#include <stdbool.h>
#include <stdarg.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>

#include <zephyr/drivers/sensor.h>

#define FALL_WORKQ_THREAD_STACK_SIZE 2048
#define FALL_WORKQ_THREAD_PRIORITY 6

#define pow2(x) ((x) * (x))
#define FALL_THRESHOLD 4


#if !DT_HAS_COMPAT_STATUS_OKAY(adi_adxl372)
#error "No adi,adxl372 compatible node found in the device tree"
#endif


K_THREAD_STACK_DEFINE(fall_workq_stack_area, FALL_WORKQ_THREAD_STACK_SIZE);

static struct k_work_q fall_work_q;
static struct k_work_delayable fall_start_work;

struct sensor_value accel[3];
double mag;
int i;
char meter[200];


const struct device *dev1 = DEVICE_DT_GET_ANY(adi_adxl372);

static double sqrt(double value)
{
	int i;
	double sqrt = value / 3;
	if (value <= 0)
	{
		return 0;
	}
	for (i = 0; i < 6; i++)
	{
		sqrt = (sqrt + value / sqrt) / 2;
	}
	return sqrt;
}

static void start_fall_work_fn(struct k_work *item)
{
	ARG_UNUSED(item);
	k_work_schedule_for_queue(&fall_work_q,
							  &fall_start_work,
							  K_MSEC(1000));

	if (sensor_sample_fetch(dev1))
	{
		printk("sensor_sample_fetch failed\n");
	}

	sensor_channel_get(dev1, SENSOR_CHAN_ACCEL_XYZ, accel);

	mag = sqrt(pow2(sensor_ms2_to_g(&accel[0])) +
			   pow2(sensor_ms2_to_g(&accel[1])) +
			   pow2(sensor_ms2_to_g(&accel[2])));
	printk("mag: %6.2f, x: %d, y: %d, z: =%d\n", mag,
		   sensor_ms2_to_g(&accel[0]),
		   sensor_ms2_to_g(&accel[1]),
		   sensor_ms2_to_g(&accel[2]));

	if (mag >= FALL_THRESHOLD)
	{
		for (i = 0; i <= mag && i < (sizeof(meter) - 1); i++)
		{
			meter[i] = '#';
		}

		meter[i] = '\0';

		printk("[FAL]: %6.2f g: %s\n", mag, meter);
		// printk("[FAL]%6.2f\n", mag);
	}
}

static void fall_api_init(void)
{
	if (!device_is_ready(dev1))
	{
		printk("Device %s is not ready\n", dev1->name);
		return;
	}

	static bool fall_api_initialized = false;
	if (fall_api_initialized)
	{
		return;
	}
	fall_api_initialized = true;
	struct k_work_queue_config fall_work_q_config = {
		.name = "fall_workq",
		.no_yield = false};
	k_work_queue_start(
		&fall_work_q,
		fall_workq_stack_area,
		K_THREAD_STACK_SIZEOF(fall_workq_stack_area),
		FALL_WORKQ_THREAD_PRIORITY,
		&fall_work_q_config);

	k_work_init_delayable(&fall_start_work, start_fall_work_fn);
	printk("fall thread : intialized successfully\n");
	k_work_schedule_for_queue(&fall_work_q, &fall_start_work, K_NO_WAIT); // start thread fall schedulation
}

void main(void)
{
	fall_api_init();
	while (true)
	{
		k_msleep(100);
	}
}